@@ -13,6 +13,7 @@
 #include "access/tuptoaster.h"
 #endif
 
+
 /*
  * get_control_data()
  *
@@ -27,7 +28,7 @@
  * pg_control data.  pg_resetxlog cannot be run while the server is running
  * so we use pg_controldata;  pg_controldata doesn't provide all the fields
  * we need to actually perform the migration, but it provides enough for
- * check mode.  We do not implement pg_resetxlog -n because it is hard to
+ * check mode.	We do not implement pg_resetxlog -n because it is hard to
  * return valid xid data for a running server.
  */
 void
@@ -53,8 +54,11 @@ get_control_data(migratorContext *ctx, ClusterInfo *cluster, bool live_check)
 	bool		got_date_is_int = false;
 	bool		got_float8_pass_by_value = false;
 	char	   *lang = NULL;
-					
-	/* Because we test the pg_resetxlog output strings, it has to be in English. */
+
+	/*
+	 * Because we test the pg_resetxlog output strings, it has to be in
+	 * English.
+	 */
 	if (getenv("LANG"))
 		lang = pg_strdup(ctx, getenv("LANG"));
 #ifndef WIN32
@@ -71,7 +75,7 @@ get_control_data(migratorContext *ctx, ClusterInfo *cluster, bool live_check)
 
 	if ((output = popen(cmd, "r")) == NULL)
 		pg_log(ctx, PG_FATAL, "Could not get control data: %s\n",
-			   strerror(errno));
+			   getErrorText(errno));
 
 	/* Only pre-8.4 has these so if they are not set below we will check later */
 	cluster->controldata.lc_collate = NULL;
@@ -101,18 +105,18 @@ get_control_data(migratorContext *ctx, ClusterInfo *cluster, bool live_check)
 
 #ifdef WIN32
 		/*
-		 *	Due to an installer bug, LANG=C doesn't work for PG 8.3.3,
-		 *	but does work 8.2.6 and 8.3.7, so check for non-ASCII output
-		 *	and suggest a minor upgrade.
+		 * Due to an installer bug, LANG=C doesn't work for PG 8.3.3, but does
+		 * work 8.2.6 and 8.3.7, so check for non-ASCII output and suggest a
+		 * minor upgrade.
 		 */
 		if (GET_MAJOR_VERSION(cluster->major_version) == 803)
 		{
 			for (p = bufin; *p; p++)
 				if (!isascii(*p))
 					pg_log(ctx, PG_FATAL,
-						"The 8.3 cluster's pg_controldata is incapable of outputting ASCII, even\n"
-						"with LANG=C.  You must upgrade this cluster to a newer version of Postgres\n"
-						"8.3 to fix this bug.  Postgres 8.3.7 and later are known to work properly.\n");
+						   "The 8.3 cluster's pg_controldata is incapable of outputting ASCII, even\n"
+						   "with LANG=C.  You must upgrade this cluster to a newer version of Postgres\n"
+						   "8.3 to fix this bug.  Postgres 8.3.7 and later are known to work properly.\n");
 		}
 #endif
 
@@ -348,7 +352,7 @@ get_control_data(migratorContext *ctx, ClusterInfo *cluster, bool live_check)
 	if (lang)
 	{
 #ifndef WIN32
-		char   *envstr = (char *) pg_malloc(ctx, strlen(lang) + 6);
+		char	   *envstr = (char *) pg_malloc(ctx, strlen(lang) + 6);
 
 		sprintf(envstr, "LANG=%s", lang);
 		putenv(envstr);
@@ -474,7 +478,7 @@ check_control_data(migratorContext *ctx, ControlData *oldctrl,
 	if (oldctrl->date_is_int != newctrl->date_is_int)
 	{
 		pg_log(ctx, PG_WARNING,
-		"\nOld and new pg_controldata date/time storage types do not match.\n");
+			   "\nOld and new pg_controldata date/time storage types do not match.\n");
 
 		/*
 		 * This is a common 8.3 -> 8.4 migration problem, so we are more
@@ -483,82 +487,22 @@ check_control_data(migratorContext *ctx, ControlData *oldctrl,
 		pg_log(ctx, PG_FATAL,
 			   "You will need to rebuild the new server with configure\n"
 			   "--disable-integer-datetimes or get server binaries built\n"
-				"with those options.\n");
+			   "with those options.\n");
 	}
 }
 
 
-/*
- * set_locale_and_encoding()
- *
- * query the database to get the template0 locale
- */
 void
-set_locale_and_encoding(migratorContext *ctx, Cluster whichCluster)
+rename_old_pg_control(migratorContext *ctx)
 {
-	PGconn	   *conn;
-	PGresult   *res;
-	int			i_encoding;
-	ControlData *ctrl = (whichCluster == CLUSTER_OLD) ?
-						&ctx->old.controldata : &ctx->new.controldata;
-	int			cluster_version = (whichCluster == CLUSTER_OLD) ?
-								ctx->old.major_version : ctx->new.major_version;
-								
-	conn = connectToServer(ctx, "template1", whichCluster);
+	char		old_path[MAXPGPATH],
+				new_path[MAXPGPATH];
 
-	/* for pg < 80400, we got the values from pg_controldata */
-	if (cluster_version >= 80400)
-	{
-		int			i_datcollate;
-		int			i_datctype;
+	prep_status(ctx, "Adding \".old\" suffix to old global/pg_control");
 
-		res = executeQueryOrDie(ctx, conn,
-								"SELECT datcollate, datctype "
-								"FROM 	pg_catalog.pg_database "
-								"WHERE	datname = 'template0' ");
-		assert(PQntuples(res) == 1);
-	
-		i_datcollate = PQfnumber(res, "datcollate");
-		i_datctype = PQfnumber(res, "datctype");
-	
-		ctrl->lc_collate = pg_strdup(ctx, PQgetvalue(res, 0, i_datcollate));
-		ctrl->lc_ctype = pg_strdup(ctx, PQgetvalue(res, 0, i_datctype));
-	
-		PQclear(res);
-	}
-	
-	res = executeQueryOrDie(ctx, conn,
-							"SELECT pg_catalog.pg_encoding_to_char(encoding) "
-							"FROM 	pg_catalog.pg_database "
-							"WHERE	datname = 'template0' ");
-	assert(PQntuples(res) == 1);
-
-	i_encoding = PQfnumber(res, "pg_encoding_to_char");
-	ctrl->encoding = pg_strdup(ctx, PQgetvalue(res, 0, i_encoding));
-
-	PQclear(res);
-		
-	PQfinish(conn);
-}
-
-
-/*
- * check_locale_and_encoding()
- *
- *	locale is not in pg_controldata in 8.4 and later so
- *	we probably had to get via a database query.
- */
-void
-check_locale_and_encoding(migratorContext *ctx, ControlData *oldctrl,
-				   ControlData *newctrl)
-{
-	if (strcmp(oldctrl->lc_collate, newctrl->lc_collate) != 0)
-		pg_log(ctx, PG_FATAL,
-			   "old and new cluster lc_collate values do not match\n");
-	if (strcmp(oldctrl->lc_ctype, newctrl->lc_ctype) != 0)
-		pg_log(ctx, PG_FATAL,
-			   "old and new cluster lc_ctype values do not match\n");
-	if (strcmp(oldctrl->encoding, newctrl->encoding) != 0)
-		pg_log(ctx, PG_FATAL,
-			   "old and new cluster encoding values do not match\n");
+	snprintf(old_path, sizeof(old_path), "%s/global/pg_control", ctx->old.pgdata);
+	snprintf(new_path, sizeof(new_path), "%s/global/pg_control.old", ctx->old.pgdata);
+	if (pg_mv_file(old_path, new_path) != 0)
+		pg_log(ctx, PG_FATAL, "Unable to rename %s to %s.\n", old_path, new_path);
+	check_ok(ctx);
 }
