@@ -14,8 +14,8 @@ static void get_db_infos(migratorContext *ctx, DbInfoArr *dbinfos,
 static void dbarr_print(migratorContext *ctx, DbInfoArr *arr,
 			Cluster whichCluster);
 static void relarr_print(migratorContext *ctx, RelInfoArr *arr);
-static void get_rel_infos(migratorContext *ctx, const char *db_name,
-			const char *db_tblspace, RelInfoArr *relarr, Cluster whichCluster);
+static void get_rel_infos(migratorContext *ctx, const DbInfo *dbinfo,
+			RelInfoArr *relarr, Cluster whichCluster);
 static void relarr_free(RelInfoArr *rel_arr);
 static void map_rel(migratorContext *ctx, const RelInfo *oldrel,
 			const RelInfo *newrel, const DbInfo *old_db,
@@ -266,8 +266,7 @@ get_db_and_rel_infos(migratorContext *ctx, DbInfoArr *db_arr, Cluster whichClust
 	get_db_infos(ctx, db_arr, whichCluster);
 
 	for (dbnum = 0; dbnum < db_arr->ndbs; dbnum++)
-		get_rel_infos(ctx, db_arr->dbs[dbnum].db_name,
-					  db_arr->dbs[dbnum].db_tblspace,
+		get_rel_infos(ctx, &db_arr->dbs[dbnum],
 					  &(db_arr->dbs[dbnum].rel_arr), whichCluster);
 
 	if (ctx->debug)
@@ -285,11 +284,10 @@ get_db_and_rel_infos(migratorContext *ctx, DbInfoArr *db_arr, Cluster whichClust
  * FirstNormalObjectId belongs to the user
  */
 static void
-get_rel_infos(migratorContext *ctx, const char *db_name,
-			  const char *db_tblspace, RelInfoArr *relarr,
-			  Cluster whichCluster)
+get_rel_infos(migratorContext *ctx, const DbInfo *dbinfo,
+			  RelInfoArr *relarr, Cluster whichCluster)
 {
-	PGconn	   *conn = connectToServer(ctx, db_name, whichCluster);
+	PGconn	   *conn = connectToServer(ctx, dbinfo->db_name, whichCluster);
 	bool		is_edb_as = (whichCluster == CLUSTER_OLD) ?
 	ctx->old.is_edb_as : ctx->new.is_edb_as;
 	PGresult   *res;
@@ -347,7 +345,7 @@ get_rel_infos(migratorContext *ctx, const char *db_name,
 	 * EDB AS installs pgagent by default via initdb. We have to ignore it,
 	 * and not migrate any old table contents.
 	 */
-			 (is_edb_as && strcmp(db_name, "edb") == 0) ?
+			 (is_edb_as && strcmp(dbinfo->db_name, "edb") == 0) ?
 			 " 	AND "
 			 "	n.nspname != 'pgagent' AND "
 	/* skip pgagent TOAST tables */
@@ -406,7 +404,7 @@ get_rel_infos(migratorContext *ctx, const char *db_name,
 		tblspace = PQgetvalue(res, relnum, i_spclocation);
 		/* if no table tablespace, use the database tablespace */
 		if (strlen(tblspace) == 0)
-			tblspace = db_tblspace;
+			tblspace = dbinfo->db_tblspace;
 		snprintf(curr->tablespace, sizeof(curr->tablespace), "%s", tblspace);
 	}
 	PQclear(res);
