@@ -147,21 +147,18 @@ quote_identifier(migratorContext *ctx, const char *s)
 
 
 /*
- * get_user_id()
+ * get_user_info()
  * (copied from initdb.c) find the current user
- *
- * on unix make sure it isn't really root. For now we only
- * use this function to get check that on *nix the user is'nt
- * root
  */
-char *
-get_user_id(migratorContext *ctx)
+int
+get_user_info(migratorContext *ctx, char **user_name)
 {
+	int	user_id;
+	
 #ifndef WIN32
 	struct passwd *pw = getpwuid(geteuid());
 
-	if (geteuid() == 0)			/* 0 is root's uid */
-		pg_log(ctx, PG_FATAL, "%s: cannot be run as root\n", ctx->progname);
+	user_id = geteuid();
 #else							/* the windows code */
 	struct passwd_win32
 	{
@@ -171,11 +168,14 @@ get_user_id(migratorContext *ctx)
 	struct passwd_win32 *pw = &pass_win32;
 	DWORD		pwname_size = sizeof(pass_win32.pw_name) - 1;
 
-	pw->pw_uid = 1;
 	GetUserName(pw->pw_name, &pwname_size);
+
+	user_id = 1;
 #endif
 
-	return pg_strdup(ctx, pw->pw_name);
+	*user_name = pg_strdup(ctx, pw->pw_name);
+
+	return user_id;
 }
 
 
